"""A module to hold the data that will be used to create PloneHelpCenter
content objects.  I'm not sure whether I can depend on relative file paths,
so we store the data in a Python module where I'm sure that it can be
imported by other modules in the same Python package.
"""

class PropertyHolder:
    """A dummy struct-like class for holding properties"""
    pass

# HelpCenter folderish object ----------------------------------------
Hc = PropertyHolder()
Hc.Id = 'hc'
Hc.Title = 'Test Help Center'
Hc.Desc = 'A help center instance for functional testing of PHC.'
Hc.Versions = ('Version 1.0', 'Version 2.0', 'Different Version 1.0')
Hc.Sections =  ('Topic1', 'Topic1: Section1', 'Topic1: Section2 - \xc3\xa1'.decode('utf8'), 'Topic2', 'Topic2: Section1', 'Topic2: Section2')

# Alternate HelpCenter folderish object ----------------------------------------
AltHc = PropertyHolder()
AltHc.Id = 'althc'
AltHc.Title = 'Alternate Test Help Center'
AltHc.Desc = 'An alternate help center instance for functional testing of PHC.'
AltHc.Versions = ('Version 1.0', 'Version 2.0', 'Different Version 1.0')

# Transitions  -------------------------------------------------------
Transition = PropertyHolder()
Transition.publish = 'publish'
Transition.submit = 'submit'
Transition.obsolete = 'mark_obsolete'

# Users --------------------------------------------------------------
User1 = PropertyHolder()
User1.Id = 'tester1'
User1.Password = 'tester1'
User1.Roles = ['Member',]

User2 = PropertyHolder()
User2.Id = 'tester2'
User2.Password = 'tester2'
User2.Roles = ['Member',]

User3 = PropertyHolder()
User3.Id = 'test_manager'
User3.Password = 'test_manager'
User3.Roles = ['Member','Manager',]

User4 = PropertyHolder()
User4.Id = 'test_reviewer'
User4.Password = 'test_reviewer'
User4.Roles = ['Member','Reviewer',]

User = PropertyHolder()
User.list = [User1, User2, User3, User4]

# HowtoFolder settings -----------------------------------------------
HowtoFolder = PropertyHolder()
HowtoFolder.Sections = ('Topic1: Section1', 'Topic1: Section2 - \xc3\xa1'.decode('utf8'), 'Topic2: Section1', 'Topic2: Section2', 'General')

# TutorialFolder settings --------------------------------------------
TutorialFolder = PropertyHolder()
TutorialFolder.Sections = ('Topic1: Section1', 'Topic1: Section2 - \xc3\xa1'.decode('utf8'), 'Topic2: Section1', 'Topic2: Section2')

# FAQFolder settings --------------------------------------------
FAQFolder = PropertyHolder()
FAQFolder.Sections = ('General', 'FAQ Section1', 'FAQ - \xc3\xa1 - Section2'.decode('utf8') )

# ErrorReferenceFolder settings --------------------------------------------
ErrorReferenceFolder = PropertyHolder()
ErrorReferenceFolder.Sections = ('General', 'Error Ref Section1 - \xc3\xa1'.decode('utf8') )

# Glossary settings --------------------------------------------
Glossary = PropertyHolder()
Glossary.Sections = ('General', )

# LinkFolder settings --------------------------------------------
LinkFolder = PropertyHolder()
LinkFolder.Sections = ('General', 'Link Section1', 'Link Section2 - \xc3\xa1'.decode('utf8'), 'Link Section3' )

# ReferenceManualFolder settings --------------------------------------------
ReferenceManualFolder = PropertyHolder()
ReferenceManualFolder.Sections = ('Topic1: Section1', 'Topic1: Section2 - \xc3\xa1'.decode('utf8'), 'Topic2: Section1', 'Topic2: Section2')

# VideoFolder settings --------------------------------------------
VideoFolder = PropertyHolder()
VideoFolder.Sections = ('General', )

# Sample Howto content 1 ---------------------------------------------
Howto1 = PropertyHolder()
Howto1.Id = 'howto1'
Howto1.Title = 'Test Howto 1'
Howto1.Summary = 'A sample Howto for functional testing of PloneHelpCenter.  A howto that is published; only in one section.'
Howto1.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe published state and is only
attached to one Section.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto1.Format = 'text/x-rst'
Howto1.Versions = ( Hc.Versions[1], )
Howto1.Sections = ( HowtoFolder.Sections[1], )
Howto1.Transition = Transition.publish
Howto1.Owner = User1

# Sample Howto content 2 ---------------------------------------------
Howto2 = PropertyHolder()
Howto2.Id = 'howto2'
Howto2.Title = 'Test Howto 2'
Howto2.Summary = 'A howto that is published in two sections'
Howto2.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe published state and is 
attached to two Howto Sections.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto2.Format = 'text/x-rst'
Howto2.Versions = ( Hc.Versions[1], )
Howto2.Sections = ( HowtoFolder.Sections[1], HowtoFolder.Sections[2], )
Howto2.Transition = Transition.publish
Howto2.Owner = User1

# Sample Howto content 3 ---------------------------------------------
Howto3 = PropertyHolder()
Howto3.Id = 'howto3'
Howto3.Title = 'Test Howto 3'
Howto3.Summary = 'A howto that is published in one section by a different owner than howto1.'
Howto3.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe published state and is only
attached to one Section.  It should be owned by a different user than
howto1.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto3.Format = 'text/x-rst'
Howto3.Versions = ( Hc.Versions[1], )
Howto3.Sections = ( HowtoFolder.Sections[1], )
Howto3.Transition = Transition.publish
Howto3.Owner = User2

# Sample Howto content 4 ---------------------------------------------
Howto4 = PropertyHolder()
Howto4.Id = 'howto4'
Howto4.Title = 'Test Howto 4'
Howto4.Summary = 'A howto that is in-progress in one section'
Howto4.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe in-progress state and is only
attached to one Section.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto4.Format = 'text/x-rst'
Howto4.Versions = ( Hc.Versions[1], )
Howto4.Sections = ( HowtoFolder.Sections[1], )
Howto4.Transition = None
Howto4.Owner = User1

# Sample Howto content 5 ---------------------------------------------
Howto5 = PropertyHolder()
Howto5.Id = 'howto5'
Howto5.Title = 'Test Howto 5'
Howto5.Summary = 'A howto that is in-progress in two sections'
Howto5.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe in-progress state and is 
attached to two Howto Sections.  It is also owned by a different user
than the other test howto that is in-progress.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto5.Format = 'text/x-rst'
Howto5.Versions = ( Hc.Versions[1], )
Howto5.Sections = ( HowtoFolder.Sections[1], HowtoFolder.Sections[2], )
Howto5.Transition = None
Howto5.Owner = User2

# Sample Howto content 6 ---------------------------------------------
Howto6 = PropertyHolder()
Howto6.Id = 'howto6'
Howto6.Title = 'Test Howto 6'
Howto6.Summary = 'A howto that is pending review in one section'
Howto6.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts in the pending state and is only
attached to one Section.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto6.Format = 'text/x-rst'
Howto6.Versions = ( Hc.Versions[1], )
Howto6.Sections = ( HowtoFolder.Sections[1], )
Howto6.Transition = Transition.submit
Howto6.Owner = User1

# Sample Howto content 7 ---------------------------------------------
Howto7 = PropertyHolder()
Howto7.Id = 'howto7'
Howto7.Title = 'Test Howto 7'
Howto7.Summary = 'A howto that is published in (a different) one section'
Howto7.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe published state and is only
attached to one Section.  The section is different from the section
to which howto1 is attached.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto7.Format = 'text/x-rst'
Howto7.Versions = ( Hc.Versions[1], )
Howto7.Sections = ( HowtoFolder.Sections[2], )
Howto7.Transition = Transition.publish
Howto7.Owner = User1

# Sample Howto content 8 ---------------------------------------------
Howto8 = PropertyHolder()
Howto8.Id = 'howto8'
Howto8.Title = 'Test Howto 8'
Howto8.Summary = 'A howto that is published in (a different) one section by a different owner than howto7.'
Howto8.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe published state and is only
attached to one Section.  The section is different from the section
to which howto1 is attached.  It is owned by a different user from howto7.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto8.Format = 'text/x-rst'
Howto8.Versions = ( Hc.Versions[1], )
Howto8.Sections = ( HowtoFolder.Sections[2], )
Howto8.Transition = Transition.publish
Howto8.Owner = User2

# Sample Howto content 9 ---------------------------------------------
Howto9 = PropertyHolder()
Howto9.Id = 'howto9'
Howto9.Title = 'Test Howto 9'
Howto9.Summary = 'A howto that is in-progress in (a different) one section'
Howto9.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe in-progress state and is only
attached to one Section.  The section is different from the section
to which howto1 is attached.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto9.Format = 'text/x-rst'
Howto9.Versions = ( Hc.Versions[1], )
Howto9.Sections = ( HowtoFolder.Sections[2], )
Howto9.Transition = None
Howto9.Owner = User1

# Sample Howto content 10 --------------------------------------------
Howto10 = PropertyHolder()
Howto10.Id = 'howto10'
Howto10.Title = 'Test Howto 10'
Howto10.Summary = 'A howto that is published in a third section'
Howto10.Body = '''
============
Howto Title
============

.. _Plone: http://www.plone.org/

This howto is a document that starts inthe published state and is only
attached to one Section.  The section is different from the section used
by howto1 or the section used by howto7.
The rest of the content below this description was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

Section 1
=========

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone_ Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories


Section 2
=========

How to get a local copy of code for a Product from the Plone_ Collective.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone_ Product.  See the list of 
folders in the CVS web interface for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you wouldd replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone_ Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you wouldd replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename

End of test document content.
'''
Howto10.Format = 'text/x-rst'
Howto10.Versions = ( Hc.Versions[1], )
Howto10.Sections = ( HowtoFolder.Sections[0], )
Howto10.Transition = Transition.publish
Howto10.Owner = User1


# Sample Howto content -----------------------------------------------
Howto = PropertyHolder()
Howto.list = [ Howto1, Howto2, Howto3, Howto4, Howto5, Howto6, Howto7, Howto8, Howto9, Howto10 ]

# Sample Tutorial Page content ---------------------------------------
Page1 = PropertyHolder()
Page1.Id = 'page1'
Page1.Title = 'Sample Tutorial Page1'
Page1.Summary = 'This tutorial page simply serves as sample content for the tutorial.'
Page1.Body = '''
Section 1
=========

The rest of the content below this point was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

This document was spawned from conversations on the #plone IRC channel. 
A new version of the PloneHelpCenter from the Plone Collective was 
undergoing testing, but some developers wanted to start working on brand 
new code.  The danger was that the new code would delay the deployment 
of some of the much-needed bug fixes and changes that were almost ready 
for deployment on plone.org.  The obivous choice was to create a branch, 
but no one seemed comfortable enough with CVS to make a branch.  I 
volunteered to make a minimal HOWTO that would explain the essential 
CVS commands for getting day to day work done, up to and including 
creating branches and merging code between branches.

I assume that you are familiar with revision control in general and
have CVS client software installed and working.  We just provide a
quick explanation of the most common commands that you will need.  While
there are some great CVS GUIs (Win CVS, Tortois CVS, Cervisia, etc.),
I will only cover the cvs command line here.  

Please see the end of this document for links to more complete CVS 
documentation and to some useful CVS tools.

General syntax
--------------

CVS command line syntax looks like this::

    cvs [global-options] command [command-options] [command-arguments]

The ``command`` is one of the CVS commands, such as checkout, update, or
commit.  To see a brief list of the available commands, use the
following option::

    cvs --help-commands 

The ``global-options`` are the same for all commands.  Each command has 
a different set of ``command-options`` and ``command-arguments``.

The most common ``global-options`` are listed here for later reference.

``-d``
    specifies the root of the CVS tree and possibly also identifies a 
    remote repository, account name, and connection method for the 
    remote repository; overrides the CVSROOT environment variable; 
    generally only used with the checkout command (after that, cvs 
    reads this information from the CVS/Root file in your personal 
    copy of the files)
``-n``
    do not execute anything that would change the disk; for many commands, 
    this options shows what would be done, but does not actually do it
``-z``
    specifies a compression level for network traffic; recommended for 
    remote repositories
'''
Page1.Format = 'text/x-rst'
Page1.Transition = Transition.publish

# Sample Tutorial Page content ---------------------------------------
Page2 = PropertyHolder()
Page2.Id = 'page2'
Page2.Title = 'Sample Tutorial Page2'
Page2.Summary = 'More sample content for the tutorial.'
Page2.Body = '''
Initial Checkout
================

The rest of the content below this point was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

The CVS command to get a local working copy of code from a central CVS 
repository is ``checkout``.  This command is normally abbreviated as ``co``.
This command will give you a copy of all of the files of a directory 
tree on your machine.  This local copy is sometimes called your 
sandbox (you get to play in your sandbox).  A checkout works on a module, 
which will just be a top-level folder in the Collective for our purposes.


You just want the latest version of a Product
---------------------------------------------

If you do not need to develop or modify code, you can use anonymous 
check out from SourceForge.  An anonymous checkout permits you to 
get and update a local copy of the code in CVS.  You do not need a 
SourceForge account.  You can also make local changes and view the 
diffs between your local code and the repository code.  You will be 
unable to submit your changes.  See also the Collective CVS page.

Enter the following line command exactly as written.  When prompted for a 
password, just press enter:: 

    cvs -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective login

Once you are logged into the pserver, you can checkout (co) the Product 
you need.  For the next command, you will need to replace *modulename* 
with the name of the module you want to checkcout.   The *modulename* is 
generally the top level folder for a Plone Product.  See the list of 
folders in the CVS web interfoce for the Collective.  For example, if you 
wanted a copy of PloneHelpCenter, you would replace *modulename* with the 
word ``PloneHelpCenter``::

    cvs -z3 -d:pserver:anonymous@cvs.sourceforge.net:/cvsroot/collective co -P modulename


You want to do development work on a Product
--------------------------------------------

Anonymous checkout does not permit you to commit changes to a Product 
in the Collective.  If you need to modify and submit code to the 
Collective, you will need to get an account on SourceForge.  Once 
you have that account, someone will need to add you to the list of 
developers for the Plone Collective project on SourceForge.  You 
will  be checking out code over a secure SSH tunnel.  SourceForge 
provides decent documentation concerning configuration of client 
software for accessing their CVS repositories over SSH.

The command to checkout code for development follows.  Remember to replace 
*developername* with your name and *modulename* with the folder of code 
you want.  For example, if you wanted to check out a copy of 
PloneHelpCenter, you would replace *modulename* with the word 
``PloneHelpCenter``::
 
    cvs -z3 -d:ext:developername@cvs.sourceforge.net:/cvsroot/collective co -P modulename
'''
Page2.Format = 'text/x-rst'
Page2.Transition = Transition.publish

# Sample Tutorial Page content ---------------------------------------
Page3 = PropertyHolder()
Page3.Id = 'page3'
Page3.Title = 'Sample Tutorial Page3'
Page3.Summary = 'Yet more sample content.  Somebody move that developer to the QA team!'
Page3.Body = '''
Related links
=============

The rest of the content below this point was basically stolen from
the CVS tutorial on Plone.org just to give us a non-trivial document.
It is one of the few reStructured Text documents for which I have the
source format.

This cheatsheet used a command line interface to CVS since it is easy
to explain and consistent across multiple platforms.  While some
operations may continue to be easier from the command line, even some
of us who generally prefer the command line like to use a graphical
CVS client when browsing file histories and looking at diffs between
revisions.  

http://www.wincvs.org/
    An excellent starting point for CVS and GUIs.  Links to multiple
    CVS client and visual diff GUIs for various platforms.
http://www.tortoisecvs.org/
    Recommended by multiple Plone developers who develop on Windows.
    This plugin enables CVS operations directly from Microsoft Windows
    file explorer. 
http://www.kde.org/apps/cervisia/
    I generally use Cervisia when I want a CVS client other than the
    command line or Emacs PCL-CVS mode under KDE on my Linux machines.

Hey, all of this CVS stuff is fascinating, right?  If you were one
of the two people who did not just fall out of your chair laughing,
then here are some links to get you started learning more.  Until
everyone stops using CVS in favor of some other revision control
software, there will be a need for developers who have a thorough
knowledge of CVS.  (Not everyone is content to be a faker!)

http://cvsbook.red-bean.com/ 
    My favorite CVS books.  Very practical.  Lots of "best practices"
    advice.  Also available in print.
https://www.cvshome.org/  
    The official home for all things CVS.
https://www.cvshome.org/docs/manual/ 
    The official CVS Manual originally by Per Cederqvist.
http://www.cmcrossroads.com/bradapp/acme/
    A nice starting point for SCM information in general, including 
    links to software other than CVS (Subversion, Perforce, ClearCase, etc.)
'''
Page3.Format = 'text/x-rst'
Page3.Transition = None

# Sample Tutorial content 1 ------------------------------------------
Tutorial1 = PropertyHolder()
Tutorial1.Id = 'tutorial1'
Tutorial1.Title = 'Test Tutorial 1'
Tutorial1.Summary = 'A sample Tutorial for functional testing of PloneHelpCenter.  A tutorial that is published; only in one section; has only a single tutorial page.'
Tutorial1.Versions = ( Hc.Versions[0], )
Tutorial1.Sections = ( TutorialFolder.Sections[1], )
Tutorial1.Pages = ( Page1, )
Tutorial1.Transition = Transition.publish
Tutorial1.Owner = User1

# Sample Tutorial content 1 ------------------------------------------
Tutorial2 = PropertyHolder()
Tutorial2.Id = 'tutorial2'
Tutorial2.Title = 'Test Tutorial 2'
Tutorial2.Summary = 'A sample Tutorial that is pending; only in one section; has three tutorial pages.'
Tutorial2.Versions = ( Hc.Versions[1], )
Tutorial2.Sections = ( TutorialFolder.Sections[1], )
Tutorial2.Pages = ( Page1, Page2, Page3, )
Tutorial2.Transition = Transition.submit
Tutorial2.Owner = User1

# Sample Tutorial content 1 ------------------------------------------
Tutorial3 = PropertyHolder()
Tutorial3.Id = 'tutorial3'
Tutorial3.Title = 'Test Tutorial 3'
Tutorial3.Summary = 'A sample Tutorial that is in-progress; only in one section; has no tutorial pages.'
Tutorial3.Versions = ( Hc.Versions[1], )
Tutorial3.Sections = ( TutorialFolder.Sections[1], )
Tutorial3.Pages = ()
Tutorial3.Transition = None
Tutorial3.Owner = User1

# Sample Tutorial content 1 ------------------------------------------
Tutorial4 = PropertyHolder()
Tutorial4.Id = 'tutorial4'
Tutorial4.Title = 'Test Tutorial 4'
Tutorial4.Summary = 'A sample Tutorial that is published; only in (a different) one section; has a couple of tutorial pages and at least one non-tutorial page piece of content.'
Tutorial4.Versions = ( Hc.Versions[1], )
Tutorial4.Sections = ( TutorialFolder.Sections[0], )
Tutorial4.Pages = ( Page1, Page2, Page3)
Tutorial4.Transition = Transition.publish
Tutorial4.Owner = User1


# Sample Tutorial content --------------------------------------------
Tutorial = PropertyHolder()
Tutorial.list = [ Tutorial1 , Tutorial2, Tutorial3, Tutorial4, ]

# Sample FAQ content 1 -----------------------------------------------
FAQ1 = PropertyHolder()
FAQ1.Id = 'faq1'
FAQ1.Title = 'What is Test FAQ 1?'
FAQ1.Question = 'This would be a long form of the question.  Sample FAQ for functional testing; published; only in one section.'
FAQ1.Answer = 'Bogus answer here.  Just a lot of text to fill some space.  ' * 3
FAQ1.Format = 'text/plain'
FAQ1.Versions = ( Hc.Versions[1], )
FAQ1.Sections = ( FAQFolder.Sections[1], )
FAQ1.Transition = Transition.publish
FAQ1.Owner = User1

# Sample FAQ content 2 -----------------------------------------------
FAQ2 = PropertyHolder()
FAQ2.Id = 'faq2'
FAQ2.Title = 'What is Test FAQ 2?'
FAQ2.Question = 'This would be a long form of the question.  Sample FAQ for functional testing; pending; only in one section.'
FAQ2.Answer = 'Bogus answer here.  Just a lot of text to fill some space.  ' * 3
FAQ2.Format = 'text/plain'
FAQ2.Versions = ( Hc.Versions[1], )
FAQ2.Sections = ( FAQFolder.Sections[1], )
FAQ2.Transition = Transition.submit
FAQ2.Owner = User1

# Sample FAQ content 3 -----------------------------------------------
FAQ3 = PropertyHolder()
FAQ3.Id = 'faq3'
FAQ3.Title = 'What is Test FAQ 3?'
FAQ3.Question = 'This would be a long form of the question.  Sample FAQ for functional testing; in-progress; only in one section.'
FAQ3.Answer = 'Bogus answer here.  Just a lot of text to fill some space.  ' * 3
FAQ3.Format = 'text/plain'
FAQ3.Versions = ( Hc.Versions[1], )
FAQ3.Sections = ( FAQFolder.Sections[1], )
FAQ3.Transition = None
FAQ3.Owner = User1

# Sample FAQ content 4 -----------------------------------------------
FAQ4 = PropertyHolder()
FAQ4.Id = 'faq4'
FAQ4.Title = 'What is Test FAQ 4?'
FAQ4.Question = 'This would be a long form of the question.  Sample FAQ for functional testing; published; only in (a different) one section.'
FAQ4.Answer = 'Bogus answer here.  Just a lot of text to fill some space.  ' * 3
FAQ4.Format = 'text/plain'
FAQ4.Versions = ( Hc.Versions[1], )
FAQ4.Sections = ( FAQFolder.Sections[2], )
FAQ4.Transition = Transition.publish
FAQ4.Owner = User1

# Sample FAQ content -------------------------------------------------
FAQ = PropertyHolder()
FAQ.list = [FAQ1, FAQ2, FAQ3, FAQ4,]


# Sample ErrorReference content --------------------------------------
ErrorReference = PropertyHolder()
ErrorReference.list = []


# Sample Glossary content --------------------------------------------
Glossary = PropertyHolder()
Glossary.list = []


# Sample LinkFolder content ------------------------------------------
Link = PropertyHolder()
Link.list = []


# Sample ReferenceManualFolder content -------------------------------
ReferenceManual = PropertyHolder()
ReferenceManual.list = []


# Sample Video content -----------------------------------------------
Video = PropertyHolder()
Video.list = []





#
# The Plone Help Center container.
#
# A Help Center is pre-populated with the following folders:
#
# /faq - contains the FAQ objects
# /how-to - contains the How-tos
# /tutorial - contains the tutorials
# /reference - contains the reference manuals
# /error - contains the error references
# /link - contains the links to other documentation
# /glossary - contains the definitions
# /video - contains video files for training/instruction
# /manual - contains reference manuals
#
# The main goals of these folders are to restrict the addable types and
# provide a sensible default view out-of-the-box, like the FAQ view.
#

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import transaction

import Products.CMFCore.permissions as CMFCorePermissions

from zope.interface import implements
from Products.PloneHelpCenter.interfaces import IHelpCenter

from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin, fti_meta_type
from AccessControl import ClassSecurityInfo
from Products.PloneHelpCenter.config import *
from Products.CMFPlone.utils import _createObjectByType

import os
from Globals import package_home

try:
    set
except NameError:
    from sets import Set as set

HCRootSchema = BaseFolderSchema + Schema((
    TextField(
        'description',
        searchable=1,
        required=1,
        accessor="Description",
        default_content_type = 'text/plain',
        allowable_content_types = ('text/plain',),
        storage=MetadataStorage(),
        widget=TextAreaWidget(
                description_msgid="phc_desc_helpcenter",
                description="Description for the Help Center.",
                label_msgid="phc_label_desc_helpcenter",
                label="Description",
                i18n_domain = "plonehelpcenter",
                rows=6,
                ),
        ),

    LinesField(
        'audiencesVocab',
        accessor='getAudiencesVocab',
        edit_accessor='getRawAudiencesVocab',
        mutator='setAudiencesVocab',
        required=0,
        languageIndependent=1,
        widget=LinesWidget(
                description_msgid="phc_audience_helpcenter",
                description="One type of audience on each line. If you leave this blank, audience information will not be used. Audience is typically 'End user', 'Developer' and similar.",
                label="Documentation Audiences",
                label_msgid="phc_label_audience_helpcenter",
                i18n_domain = "plonehelpcenter",
                ),
        ),

    LinesField(
        'versionsVocab',
        accessor='getVersionsVocab',
        edit_accessor='getRawVersionsVocab',
        mutator='setVersionsVocab',
        required=0,
        languageIndependent=1,
        widget=LinesWidget(
                description_msgid="phc_version_helpcenter",
                description="One version on each line, if you're documenting different versions of software. If you leave this blank, version information will not be used.",
                label="Versions",
                label_msgid="phc_label_version_helpcenter",
                i18n_domain = "plonehelpcenter",
                ),
        ),

    LinesField(
        'currentVersions',
        required=0,
        multiValued=1,
        languageIndependent=1,
        vocabulary='getVersionsVocab',
        widget=MultiSelectionWidget(
            description_msgid="phc_current_versions_helpcenter",
            description="Readers will be informed when content relates to versions not in this list, as it may be outdated.",
            label="Current Versions",
            label_msgid="phc_label_current_versions_helpcenter",
            i18n_domain = "plonehelpcenter",
            ),
        ),

    LinesField(
        'sectionsVocab',
        accessor='getSectionsVocab',
        edit_accessor='getRawSectionsVocab',
        mutator='setSectionsVocab',
        widget=LinesWidget(
           label="Sections",
           description="One section on each line. Used for grouping items. May be overriden in individual help center folders.",
           description_msgid = "phc_topsections_vocab",
           label_msgid = "phc_label_sections-vocab",
           i18n_domain="plonehelpcenter",
           rows=6,
           )
        ),

    TextField(
        'rights',
        accessor="Rights",
        widget=TextAreaWidget(
                label='Copyright',
                description="Copyright info for all content in the helpcenter.",
                label_msgid="phc_label_copyrights_helpcenter",
                description_msgid="phc_copyrights_helpcenter",
                i18n_domain="plonehelpcenter"
                ),
        ),

    BooleanField(
        'constrainSearches',
        default=1,
        required=0,
        languageIndependent=1,
        widget=BooleanWidget(
                label='Constrain Searches',
                description="""Constrain the results of Help Center searches to this document area.
                    Turn this off to search the entire site.
                    This affects only the Help Center's search facility, not the global search.
                """,
                label_msgid="phc_label_constrainSearches_helpcenter",
                description_msgid="phc_constrainSearches_helpcenter",
                i18n_domain="plonehelpcenter"
            ),
        ),
    ),)


class HelpCenter(BrowserDefaultMixin, OrderedBaseFolder):
    """
        Help Center Top Container
    """

    __implements__ = (BrowserDefaultMixin.__implements__,
        OrderedBaseFolder.__implements__)

    implements(IHelpCenter)

    schema = HCRootSchema

    content_icon = 'helpcenter_icon.gif'

    archetype_name = 'Help Center'
    meta_type = 'HelpCenter'
    filter_content_types = 1
    _at_rename_after_creation = True

    allowed_content_types = ('HelpCenterFAQFolder',
                             'HelpCenterHowToFolder',
                             'HelpCenterTutorialFolder',
                             'HelpCenterReferenceManualFolder',
                             'HelpCenterLinkFolder',
                             'HelpCenterErrorReferenceFolder',
                             'HelpCenterGlossary',
                             'HelpCenterInstructionalVideoFolder',
                             'HelpCenterKnowledgeBase',
                             )

    security = ClassSecurityInfo()

    default_view = 'helpcenter_view'
    suppl_views = ('helpcenter_topicview', 'helpcenter_topicview_main',)

    actions = (
        {'id'          : 'view',
         'name'        : 'View',
         'action'      : 'string:${object_url}',
         'permissions' : (CMFCorePermissions.View,)
         },
        {'id'          : 'edit',
         'name'        : 'Edit',
         'action'      : 'string:${object_url}/edit',
         'permissions' : (CMFCorePermissions.ModifyPortalContent,),
         },
        {'id'          : 'metadata',
         'name'        : 'Properties',
         'action'      : 'string:${object_url}/properties',
         'permissions' : (CMFCorePermissions.ModifyPortalContent,),
         },
        {'id'          : 'local_roles',
         'name'        : 'Sharing',
         'action'      : 'string:${object_url}/sharing',
         'permissions' : (CMFCorePermissions.ManageProperties,),
         },
        {'id'          : 'stats',
         'name'        : 'Statistics',
         'action'      : 'string:${object_url}/stats',
         'permissions' : (CMFCorePermissions.ManageProperties,),
         },
        )

    aliases = {
        '(Default)'  : '(dynamic view)',
        'view'       : '(selected layout)',
        'index.html' : '',
        'edit'       : 'base_edit',
        'properties' : 'base_metadata',
        'sharing'    : 'folder_localrole_form',
        'stats'      : 'phc_stats',
        'gethtml'    : '',
        'mkdir'      : '',
        }

    def initializeArchetype(self, **kwargs):
        """Pre-populate the help center folder with its basic folders. Also
        add a standard how-to-use-this-resource help page. The body text of
        this is found in docs/PHCManual.stx.
        """
        OrderedBaseFolder.initializeArchetype(self,**kwargs)

        # Set right_slots to empty
        self.manage_changeProperties(right_slots=[])

        if 'faq' not in self.objectIds():
            _createObjectByType('HelpCenterFAQFolder',self, 'faq')
            obj = self['faq']
            obj.setTitle(self.translate(
                    msgid='phc_faq_title',
                    domain='plonehelpcenter',
                    default='FAQs'))
            obj.setDescription(self.translate(
                    msgid='phc_faq_description',
                    domain='plonehelpcenter',
                    default='Frequently Asked Questions'))
            obj.reindexObject()

        if 'how-to' not in self.objectIds():
            _createObjectByType('HelpCenterHowToFolder',self, 'how-to')
            obj = self['how-to']
            obj.setTitle(self.translate(
                    msgid='phc_howto_title',
                    domain='plonehelpcenter',
                    default='How-tos'))
            self['how-to'].setDescription(self.translate(
                    msgid='phc_howto_description',
                    domain='plonehelpcenter',
                    default='Step-by-step instructions.'))
            obj.reindexObject()

        if 'tutorial' not in self.objectIds():
            _createObjectByType('HelpCenterTutorialFolder',self, 'tutorial')
            obj = self['tutorial']
            obj.setTitle(self.translate(
                    msgid='phc_tutorial_title',
                    domain='plonehelpcenter',
                    default='Tutorials'))
            obj.setDescription(self.translate(
                    msgid='phc_tutorial_description',
                    domain='plonehelpcenter',
                    default='Detailed tutorials.'))
            obj.reindexObject()

        if 'manual' not in self.objectIds():
            _createObjectByType('HelpCenterReferenceManualFolder',self, 'manual')
            obj = self['manual']
            obj.setTitle(self.translate(
                    msgid='phc_referencemanual_title',
                    domain='plonehelpcenter',
                    default='Reference Manuals'))
            obj.setDescription(self.translate(
                    msgid='phc_referencemanual_description',
                    domain='plonehelpcenter',
                    default='Reference manuals for larger projects.'))
            obj.reindexObject()

        # XXX: Video type is not yet finished
        # if 'video' not in self.objectIds():
        #    self.invokeFactory('HelpCenterInstructionalVideoFolder','video')
        #    obj = self['video']
        #    obj.setTitle(self.translate(
        #            msgid='phc_video_title',
        #            domain='plonehelpcenter',
        #            default='Videos'))
        #    obj.setDescription(self.translate(
        #            msgid='phc_video_description',
        #            domain='plonehelpcenter',
        #            default='Instructional videos.'))
        #    obj.reindexObject()

        if 'error' not in self.objectIds():
            _createObjectByType('HelpCenterErrorReferenceFolder',self, 'error')
            obj = self['error']
            obj.setTitle(self.translate(
                    msgid='phc_errorreference_title',
                    domain='plonehelpcenter',
                    default='Error References'))
            obj.setDescription(self.translate(
                    msgid='phc_errorreference_description',
                    domain='plonehelpcenter',
                    default='Error reference section.'))
            obj.reindexObject()

        if 'link' not in self.objectIds():
            _createObjectByType('HelpCenterLinkFolder',self, 'link')
            obj = self['link']
            obj.setTitle(self.translate(
                    msgid='phc_links_title',
                    domain='plonehelpcenter',
                    default='Links'))
            obj.setDescription(self.translate(
                    msgid='phc_links_description',
                    domain='plonehelpcenter',
                    default='Links section.'))
            obj.reindexObject()

        if 'glossary' not in self.objectIds():
            _createObjectByType('HelpCenterGlossary',self, 'glossary')
            obj = self['glossary']
            obj.setTitle(self.translate(
                    msgid='phc_glossary_title',
                    domain='plonehelpcenter',
                    default='Glossary Definitions'))
            obj.setDescription(self.translate(
                    msgid='phc_glossary_description',
                    domain='plonehelpcenter',
                    default='Glossary of terms.'))
            obj.reindexObject()

        transaction.savepoint(optimistic=True)

        # Add how-to-use-this-resource how-to
        if MANUAL_ID not in self['how-to'].objectIds():
            manualPath = os.path.join(package_home(GLOBALS), MANUAL_PATH)
            try:
                manualFile = open(manualPath, 'r')
                manualText = manualFile.read()
                manualFile.close()
            except IOError:
                # Silently ignore if we can't open/read the file
                pass
            else:
                # XXX: With the factory on, this causes an AttributeError

                self['how-to'].invokeFactory('HelpCenterHowTo', MANUAL_ID)
                manual = getattr(self['how-to'], MANUAL_ID)
                manual.setTitle(MANUAL_TITLE)
                manual.setDescription(MANUAL_DESCRIPTION)
                manual.setSections([MANUAL_SECTION])
                manual.setText(manualText, mimetype = MANUAL_MIMETYPE)
                manual.reindexObject()

                # Publish it
                wftool = getToolByName(self, 'portal_workflow')
                try:
                    wftool.doActionFor(manual, 'publish')
                except WorkflowException:
                    wftool.doActionFor(manual, 'submit')


    ##############
    # the following methods are meant to be used in an enclosed object
    # that is invoking the method by aquisition.

    security.declareProtected(CMFCorePermissions.View, 'getPHCObject')
    def getPHCObject(self):
        """return the enclosing PHC object"""

        return self


    security.declareProtected(CMFCorePermissions.View, 'getPHCUrl')
    def getPHCUrl(self):
        """return the enclosing PHC object URL"""

        return self.absolute_url()


    security.declareProtected(CMFCorePermissions.View, 'getPHCPath')
    def getPHCPath(self):
        """return the enclosing PHC object path as a string"""

        return '/'.join(self.getPhysicalPath())


registerType(HelpCenter, PROJECTNAME)



#
# Tests for formats other than plain-text in the editing views
# 
# http://plone.org/products/plonehelpcenter/issues/99
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from plone.app.controlpanel.markup import IMarkupSchema

from Products.PloneHelpCenter.tests import PHCTestCase


class TestDescriptionFormats(PHCTestCase.PHCTestCase):
    """
    Test that the Description field only advertises itself as
    acception text/plain
    """

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        
        # In the Markup Control Panel, choose more than one text
        # format from the "Alternative Formats" choices
        markup_data = IMarkupSchema(self.portal)
        markup_data.allowed_types = (
            'text/html','text/plain','text/restructured'
        )
        
        self._createFAQ(self.folder.hc.faq, 'f')
        self.faq = self.folder.hc.faq.f
        self._createHowto( getattr(self.folder.hc, 'how-to'), 'howto1' )
        self.howto = getattr(self.folder.hc, 'how-to').howto1
        self._createTutorial(self.folder.hc.tutorial, 't')
        self.tutorial = self.folder.hc.tutorial.t
        self._createLink(self.folder.hc.link, 'l')
        self.link = self.folder.hc.link.l
        self._createErrorReference(self.folder.hc.error, 'e')
        self.errorRef = self.folder.hc.error.e
        # phew, there are a lot of content types in PHC ...
    
    def testAllowableContentTypes(self):
        for obj in [
            self.faq, self.howto, self.tutorial, self.link, self.errorRef,
        ]:
            field = obj.getField('description')
            self.assertEqual(
                tuple(field.getAllowedContentTypes(obj)), ('text/plain',)
            )


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestDescriptionFormats))
    return suite

if __name__ == '__main__':
    framework()



#
# Tests for ErrorReference types in the PHC
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneHelpCenter.tests import PHCTestCase


class TestErrorReference(PHCTestCase.PHCTestCase):
    """General tests for ErrorReference Folder and ErrorReference objects."""

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self._createErrorReference(self.folder.hc.error, 'e')
        self.errorRef = self.folder.hc.error.e

    def testInitialTitle(self):
        # Test that the default title is correct.
        self.assertEqual(self.errorRef.title, 'Default Testing Title')

    def testInitialDescription(self):
        # Test that the default description is correct.
        self.assertEqual(self.errorRef.Description(), 'An error reference for unit tests.')

    def testInitialSections(self):
        # Test that the default section list is correct.
        self.assertEqual(self.errorRef.getSections(), ('General',))

    def testInitialVersions(self):
        # Test that the default version list is correct.
        self.assertEqual(self.errorRef.getVersions(), ('Version 2.0',))

    def testVersionsonErrorReference(self):
        versions = ('1.0','2.0','Strange version')
        self.folder.hc.setVersionsVocab(versions)
        newVersions = self.errorRef.getVersionsVocab()
        self.assertEqual(newVersions, versions)



def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestErrorReference))
    return suite

if __name__ == '__main__':
    framework()



try:
    from Products.contentmigration.migrator import InlineFieldActionMigrator, BaseInlineMigrator
    from Products.contentmigration.walker import CustomQueryWalker
    haveContentMigrations = True
except ImportError:
    haveContentMigrations = False
    
import types

from StringIO import StringIO
from Products.CMFCore.utils import getToolByName
from Products.Archetypes import transaction
from Products.Archetypes.BaseUnit import BaseUnit
from Products.CMFPlone.utils import safe_hasattr

from Acquisition import aq_base

def mergeKeywords(obj, value, **kwargs):
    if type(value) == type(()):
        value = list(value)
    elif type(value) == type('') or type(value) == type(u''):
        value = [value]
    
    newValue = []
    for v in value:
        if type(v) != type('') and type(v) != type(u''):
            continue
        if ',' in v:
            for i in v.split(','):
                newValue.append(i.strip())
        else:
            newValue.append(v.strip())
        
    md = getattr(aq_base(obj), '_md', None)
    if md:
        oldKeywords = md.get('subject', [])
        for kw in oldKeywords:
            if kw not in newValue:
                newValue.append(kw)

    return newValue

def v0_8_to_v0_9(self, out):
    """Migrate from 0.8 to 0.9
    """
    
    if not haveContentMigrations:
        print >> out, "WARNING: Install contentmigrations to be able to migrate from 0.8 to 0.9"
        return

    class HelpCenterMigrator(InlineFieldActionMigrator):
        src_portal_type = src_meta_type = ('HelpCenter')
        fieldActions = ({ 'fieldName' : 'versions_vocab',
                          'newFieldName' : 'versionsVocab',
                        },
                        { 'fieldName' : 'audiences_vocab',
                          'newFieldName' : 'audiencesVocab',
                        },

                        )

    class ContainerMigrator(InlineFieldActionMigrator):
        src_portal_type = src_meta_type = ('HelpCenterGlossary',
                                           'HelpCenterFAQFolder',
                                           'HelpCenterTutorialFolder',
                                           'HelpCenterReferenceManualFolder',
                                           'HelpCenterInstructionalVideoFolder',
                                           'HelpCenterLinkFolder',
                                           'HelpCenterHowToFolder',
                                           'HelpCenterErrorReferenceFolder',)
        fieldActions = ({ 'fieldName' : 'sections_vocab',
                          'newFieldName' : 'sectionsVocab',
                        },
                        )
                        
    class ItemMigrator(InlineFieldActionMigrator):
        src_portal_type = src_meta_type = ('HelpCenterDefinition', 
                                           'HelpCenterFAQ', 
                                           'HelpCenterTutorial',
                                           'HelpCenterReferenceManual',
                                           'HelpCenterInstructionalVideo',
                                           'HelpCenterLink',
                                           'HelpCenterHowTo',
                                           'HelpCenterErrorReference',)
        fieldActions = ({ 'fieldName' : 'referenced_items',
                          'newFieldName' : 'relatedItems',
                        },
                        { 'fieldName' : 'related_keywords',
                          'newFieldName' : 'subject',
                          'transform' : mergeKeywords,
                        },
                        )

    portal = getToolByName(self, 'portal_url').getPortalObject()

    # Migrate 
    walker = CustomQueryWalker(portal, HelpCenterMigrator, query = {})
    transaction.savepoint(optimistic=True)
    print >> out, "Migrating fields in help center root"
    walker.go()
    
    walker = CustomQueryWalker(portal, ContainerMigrator, query = {})
    transaction.savepoint(optimistic=True)
    print >> out, "Migrating fields in containers"
    walker.go()

    walker = CustomQueryWalker(portal, ItemMigrator, query = {})
    transaction.savepoint(optimistic=True)
    print >> out, "Migrating fields in items"
    walker.go()

    print >> out, "- You may also want to update your catalog in portal_catalog"
    print >> out, "- You may also want to update role mappings in portal_workflow"
    
def migrate(self):
    """Run migrations
    """
    out = StringIO()
    print >> out, "Starting PHC migration"
    v0_8_to_v0_9(self, out)
    print >> out, "PHC migrations finished"
    return out.getvalue()


try:
    from Products.CMFCore.permissions import AddPortalContent
    from Products.CMFCore.permissions import setDefaultRoles
except ImportError:    
    from Products.CMFCore.CMFCorePermissions import AddPortalContent
    from Products.CMFCore.CMFCorePermissions import setDefaultRoles
from Products.Archetypes.public import DisplayList

ADD_CENTER_PERMISSION = 'PloneHelpCenter: Add Plone Help Center'
ADD_CONTENT_PERMISSION = AddPortalContent
ADD_HELP_AREA_PERMISSION = 'PloneHelpCenter: Add Help Center Area'
ADD_DOCUMENTATION_PERMISSION = 'PloneHelpCenter: Add Documentation'

# Let members by default be allowed to add documentation, and managers only
# be able to add new documentation areas
setDefaultRoles(ADD_CENTER_PERMISSION, ('Manager',))
setDefaultRoles(ADD_HELP_AREA_PERMISSION, ('Manager', 'Owner',))
setDefaultRoles(ADD_DOCUMENTATION_PERMISSION, ('Manager', 'Member',))

PROJECTNAME = "PloneHelpCenter"
SKINS_DIR = 'skins'

GLOBALS = globals()

# If plone.memoize is available, the minutes to cache
# the results of expensive queries that help build
# the PHC top page.
# Set to zero for no caching.
# Note that editing the PHC object will force cache 
# invalidate.
CACHE_MINUTES = 30

DEFAULT_CONTENT_TYPES = {
    'default_output_type': 'text/html',
    'default_content_type': 'text/html',
    'allowable_content_types': ('text/plain',
                                'text/restructured',
                                'text/html',
                                'text/structured',)
    }

# A bug in CMF 2.1 prevents setting default discussion.
# If this gets fixed, we can turn this back on here
# and in the individual content types.
# IS_DISCUSSABLE = True

# The Rights DC metadata attribute (copyright) can be disabled on each content
# item, with the value for all items being set at the root HelpCenter level.
# Set GLOBAL_RIGHTS to 0 to turn this feature off.
GLOBAL_RIGHTS = 1

# here you can specify which types are allowed as references.
REFERENCEABLE_TYPES = ('HelpCenterFAQ',
    'HelpCenterDefinition',
    'HelpCenterTutorial',
    'HelpCenterErrorReference',
    'HelpCenterHowTo',
    'HelpCenterLink',
    'HelpCenterReferenceManual',
    'HelpCenterReferenceManualSection',
    'HelpCenterReferenceManualPage',
    'HelpCenterInstructionalVideo',    
    'HelpCenterKnowledgeBase',
)

IMAGE_SIZES = {
    'preview': (400, 400),
    'thumb': (128, 128),
    'tile': (64, 64),
    'icon': (32, 32),
    'listing': (16, 16),
    }

# Path to HelpCenter How-to
MANUAL_PATH        = 'doc/PHCManual.stx'
MANUAL_MIMETYPE    = 'text/structured'
MANUAL_ID          = 'use-help-center'
MANUAL_TITLE       = 'How to use this resource'
MANUAL_DESCRIPTION = 'A brief description of how you use and contribute to the Help Center.'
MANUAL_SECTION     = 'General'

# These are the types for which we'll support topic searches and views
TOPIC_VIEW_TYPES = ['HelpCenterTutorial','HelpCenterReferenceManual','HelpCenterHowTo']



from setuptools import setup, find_packages

version = open("Products/PloneHelpCenter/version.txt").read().strip()

setup(name='Products.PloneHelpCenter',
      version=version,
      description="A simple help-desk style documentation product for Plone.",
      long_description= open("Products/PloneHelpCenter/README.txt").read() + '\n' +
                        open("Products/PloneHelpCenter/INSTALL.txt").read() + '\n' +
                        open("Products/PloneHelpCenter/CREDITS.txt").read(),
      classifiers=[
        "Framework :: Plone",
        "Framework :: Zope2",
        "Programming Language :: Python",
      ],
      keywords='Zope CMF Plone help documentation',
      author='Plone Documentation Team',
      author_email='plone-docs@lists.sourceforge.net',
      maintainer='Steve McMahon',
      maintainer_email='steve@dcn.org',
      url='http://svn.plone.org/svn/collective/Products.PloneHelpCenter/trunk',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      download_url='http://plone.org/products/plonehelpcenter',
      install_requires=[
        'setuptools',
        'plone.i18n',
      ],
)



## Script(Python) "referencemanual_getSections"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Create the list of sections to print on the single page view of a manual 
##

# Returns a list of dicts, containing:
#
#   item     : The object of the section or page
#   hasBody  : True if this item has body text(i.e. it's a page)
#   number   : The numbering of the section(e.g. "1." or "2.")
#   contents : A list of sub-sections and contained pages
#
# The list of contained pages is another list of dicts, containing:
#
#   item     : The object of the section or page
#   hasBody  : True if this item has body text(i.e. it's a page)
#   number   : The numbering of the section(e.g. "1.2." or "2.3.1")
#   depth    : The depth of the section. Second-level sections are depth 2
#

filter = {'portal_type' : ['HelpCenterReferenceManualSection',
                           'HelpCenterReferenceManualPage']}

def getSubsections(section, depth, numbering):
    """Find all pages and subsections of the given top level, traversing the
    entire tree and flattening it to a list, depth first.
    """

    # Short circuit if we hit a page(no contents)
    if not section.isPrincipiaFolderish:
        return []

    items = []
    idx = 1
    for item in section.getFolderContents(contentFilter = filter):
        sectionNumber = "%s%d." % (numbering, idx,)
        hasBody = hasattr(item, 'getBody')
        items.append({'item'    : item,
                      'hasBody' : hasBody,
                      'number'  : sectionNumber,
                      'depth'   : depth})

        # Add sub-items of this section(shortcircuits if it was a page)
        items.extend(getSubsections(item, depth + 1, sectionNumber))

        idx += 1

    return items

# Begin traversal

sections = []
idx = 1
for topLevel in context.getFolderContents(contentFilter = filter):
    sectionNumber = "%s." % (idx,)
    hasBody = hasattr(topLevel, 'getBody')
    sections.append({'item'     : topLevel,
                      'hasBody'  : hasBody,
                      'number'   : sectionNumber,
                      'contents' : getSubsections(topLevel, 2, sectionNumber)})
    idx +=1

return sections


#
# Tests for Howto types in the PHC
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneHelpCenter.tests import PHCTestCase


class TestHowto(PHCTestCase.PHCTestCase):
    """General tests for Howto folder and HOWTO objects."""

    def afterSetUp(self):
        PHCTestCase.PHCTestCase.afterSetUp(self)
        self._createHowto( getattr(self.folder.hc, 'how-to'), 'howto1' )
        self.howto = getattr(self.folder.hc, 'how-to').howto1

    def testVersionsHowto(self):
        versions = ('1.0','2.0','Strange version')
        self.folder.hc.setVersionsVocab(versions)
        newVersions = self.howto.getVersionsVocab()
        self.assertEqual(newVersions, versions)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestHowto))
    return suite

if __name__ == '__main__':
    framework()



## Script (Python) "migratewiki2phc"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Migrates the How-To Wiki pages to PloneHelpCenter
##
newf = context.documentation2.howto
change_owner = context.plone_utils.changeOwnershipOf
transition = context.portal_workflow.doActionFor

def mkNiceId(id):
    """
    """
    assert(isinstance(id, str))
    newId = []
    for s in id:
        if s.isupper():
            newId.append('-'+s.lower())
        else:
            newId.append(s)
    strid = ''.join(newId)
    if strid.startswith('-'):
        strid = strid[1:]
    return strid

for old in context.documentation.howto.contentValues():
        newid=mkNiceId(old.getId())
        newf.invokeFactory('HelpCenterHowTo', newid)
        new = newf[newid]
        try:
            new.setBody(old.read())
        except:
            print "ERROR setting body for %s" % old.getId()
        new.setContentType('text/structured')
        new.setTitle(old.Title())
        new.setRelated_keywords((old.getId(),))
        try:
            change_owner(new, old.Creator(), 0)
        except:
            print "ERROR setting owner for %s" % old.getId()
        transition(new, 'submit')
        print "converted %s" % old.absolute_url()

return printed



from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from DateTime import DateTime
from Acquisition import aq_inner, aq_parent, aq_base, aq_chain
from Products.Archetypes.utils import shasattr
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces.Discussions import DiscussionResponse as IDiscussionResponse
try:
    from Products.CMFCore.permissions import ReplyToItem
except ImportError:
    from Products.CMFCore.CMFCorePermissions import ReplyToItem
from Products.CMFDefault.DiscussionItem import DiscussionItem, DiscussionItemContainer
from Products.CMFDefault.DiscussionTool import DiscussionTool
from config import *

PATCH_PREFIX = '_monkey_'

__refresh_module__ = 0

def monkeyPatch(originalClass, patchingClass):
    #print 'monkeyPatch', originalClass.__name__, patchingClass.__name__
    """Monkey patch original class with attributes from new class
       (Swiped from SpeedPack -- thanks, Christian Heimes!)
    
    * Takes all attributes and methods except __doc__ and __module__ from patching class
    * Safes original attributes as _monkey_name
    * Overwrites/adds these attributes in original class
    """
    for name, newAttr in patchingClass.__dict__.items():
        # don't overwrite doc or module informations
        if name not in ('__doc__', '__module__'):
            # safe the old attribute as __monkey_name if exists
            # __dict__ doesn't show inherited attributes :/
            orig = getattr(originalClass, name, None)
            if orig:
                stored_orig_name = PATCH_PREFIX + name
                stored_orig = getattr(originalClass, stored_orig_name, None)
                # don't double-patch on refresh!
                if stored_orig is None:
                    setattr(originalClass, stored_orig_name, orig)
            # overwrite or add the new attribute
            setattr(originalClass, name, newAttr)


class PatchedDiscussionItemContainer:
    #
    #   Discussable interface
    #

    security = ClassSecurityInfo()

    security.declareProtected(ReplyToItem, 'createReply')
    def createReply( self, title, text, Creator=None ):
        """Create a reply in the proper place
        """
        container = self._container

        id = int(DateTime().timeTime())
        while self._container.get( str(id), None ) is not None:
            id = id + 1
        id = str( id )

        item = DiscussionItem( id, title=title, description=title )
        item._edit( text_format='structured-text', text=text )

        if Creator:
            item.creator = Creator

        item.__of__( self ).indexObject()

        item.setReplyTo( self._getDiscussable() )

        item.__of__(self).notifyWorkflowCreated()  # added with the patch - should be fixed in CMF 1.4.8+

        self._container[ id ] = item

        return id

class PatchedDiscussionTool:

    security = ClassSecurityInfo()

    # This patch fixes CMF collector issue 314

    def getDiscussionFor(self, content):
        """ Get DiscussionItemContainer for content, create it if necessary.
        """
        if not self.isDiscussionAllowedFor( content ):
            raise DiscussionNotAllowed

        if not IDiscussionResponse.isImplementedBy(content):
            # Discussion Items use the DiscussionItemContainer object of the
            # related content item, so talkback needs to be acquired
            talkback = getattr( aq_base(content), 'talkback', None )
            if not talkback:
                self._createDiscussionFor( content )
        return content.talkback # make sure to return fully wrapped content object

try:
    from Products.CMFCore.utils import _globals as cmfcore_globals
except ImportError:
    from Products.CMFCore import cmfcore_globals
from App.Common import package_home
from os.path import join
import zLOG

x = []
CMF_VERSION = 'Unknown'
try:
    file = join(package_home(cmfcore_globals), 'version.txt')
    CMF_VERSION = open(file, 'r').read().strip()
    version = CMF_VERSION.strip()
    if version.lower().startswith('cmf-'):
        version = version[4:]
    filtered = ''
    for v in version:
        if v in ['0','1','2','3','4','5','6','7','8','9','.']:
            filtered += v
        else:
            break
    x = [int(x) for x in filtered.split('.')]
except IOError:
    # couldnt find file, oh well
    pass
except ValueError:
    # couldnt make sense of the version number
    pass
if x <= [1,4,7]:
    zLOG.LOG('PloneHelpCenter', zLOG.INFO, 'Monkey patching CMFDefault.DiscussionItem')
    monkeyPatch(DiscussionItemContainer, PatchedDiscussionItemContainer)
    monkeyPatch(DiscussionTool, PatchedDiscussionTool)



from zope.interface import implements
from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions

from Products import ATContentTypes
from Products.ATContentTypes.interfaces import IATDocument
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.PloneHelpCenter.config import *
from Products.PloneHelpCenter.content.PHCContent import HideOwnershipFields
from Products.PloneHelpCenter.interfaces import IHelpCenterMultiPage, IHelpCenterContent


HelpCenterReferenceManualPageSchema = ATContentTypes.content.document.ATDocumentSchema.copy()
HideOwnershipFields(HelpCenterReferenceManualPageSchema)
# Support specified content formats
for key, attr in DEFAULT_CONTENT_TYPES.items():
    setattr(HelpCenterReferenceManualPageSchema['text'], key, attr)


class HelpCenterReferenceManualPage(ATContentTypes.content.document.ATDocumentBase):
    """Part of a reference manual."""
    
    __implements__ = (ATContentTypes.content.document.ATDocumentBase.__implements__,
                      IATDocument)

    schema = HelpCenterReferenceManualPageSchema

    portal_type = meta_type = 'HelpCenterReferenceManualPage'
    archetype_name = 'Manual Page'

    security = ClassSecurityInfo()


    # Satisfy metadata requirements for items with deleted ownership.
    # It would be great to do this in a mixin or adapter,
    # but the structure of Archetypes prevents that.
    
    security.declareProtected(CMFCorePermissions.View, 'Rights')
    def Rights(self):
        """ get from parent """
        return aq_inner(self).aq_parent.Rights()
    
    security.declareProtected(CMFCorePermissions.View, 'Creators')
    def Creators(self):
        """ get from parent """
        return aq_inner(self).aq_parent.Creators()
    
    security.declareProtected(CMFCorePermissions.View, 'Contributors')
    def Contributors(self):
        """ get from parent """
        return aq_inner(self).aq_parent.Contributors()
        
    security.declareProtected(CMFCorePermissions.View, 'listCreators')
    def listCreators(self):
        """ List Dublin Core Creator elements - resource authors.
        """
        return self.Creators()
        

registerType(HelpCenterReferenceManualPage, PROJECTNAME)




## Script (Python) "queryForStaleItems"
##title=Final query for "stale" HelpCenter items, i.e. unchanged for a specified interval and type
##parameters=type,stale_interval

# Expects two additional items on the query-string:
#  type is the chosen portal_type, e.g. 'HelpCenterFAQ'
#  stale_interval is how old the items need to be, e.g. '3_months'

# Construct a catalog query using the specified interval (count back from
# the current date-time) and return the list of matching result objects
thresholdValues = {
    '1_day': 1,
    '1_week': 7,
    '2_weeks': 14,
    '1_month': 30,
    '3_months': 90,
    '6_months': 180,
    '1_year': 365,
}

stale_threshold_time = DateTime() - thresholdValues[ stale_interval ]
# print "Items unchanged since %s" % stale_threshold_time

time_string = '%s' % stale_threshold_time
date_string = time_string.split(' ')[0]

context.REQUEST.response.redirect(
        "search?review_state=published&portal_type:list=%s&modified:date=%s&modified_usage=range:max&submit=Search"
        % (type, date_string)
)



from zope.interface import implements
from AccessControl import ClassSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

import Products.CMFCore.permissions as CMFCorePermissions
from Products.CMFCore.utils import getToolByName

from Products.PloneHelpCenter.config import *
from Products.CMFPlone.utils import base_hasattr
from Products.PloneHelpCenter.interfaces import IHelpCenterContent

try:
    set
except NameError:
    from sets import Set as set

# Get HistoryAwareMixin on all our types:

try:
    from Products.Archetypes.HistoryAware import HistoryAwareMixin
except ImportError:
    try:
        from Products.ATContentTypes.HistoryAware import HistoryAwareMixin
    except ImportError:
        class HistoryAwareMixin:
            """Dummy class when we can't find the real McCoy"""
            
            __implements__ =()
            actions        =()


class PHCContentMixin:
    """ Provide vocabulary and default methods for HelpCenterItemSchemaNarrow """

    implements(IHelpCenterContent)

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.View, 'getVersionsVocab')
    def getVersionsVocab(self):
        """Get version vocabulary"""
        if base_hasattr(self.aq_parent, 'getVersionsVocab'):
            return self.aq_parent.getVersionsVocab()
        else:
            return ()

    security.declareProtected(CMFCorePermissions.View, 'getSectionsVocab')
    def getSectionsVocab(self):
        """Get sections vocabulary"""
        if base_hasattr(self.aq_parent, 'getSectionsVocab'):
            return self.aq_parent.getSectionsVocab()
        else:
            return ()

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'SetSections')
    def setSections(self, values):
        """set sections"""

        # The sections field may be in use with a "topic : subtopic" pattern.
        # If so, make sure that the 'topic' is set as a separate item
        # so that we'll be able to look it up by topic as well as
        # by "topic : subtopic"

        valueSet = set(values)
        for s in values:
            pos = s.find(':')
            if pos >= 0:
                valueSet.add( s[:pos].strip() )
        self.sections = tuple(valueSet)


    security.declareProtected(CMFCorePermissions.View, 'getAudiencesVocab')
    def getAudiencesVocab(self):
        """Get audiences vocabulary"""
        if base_hasattr(self.aq_parent, 'getAudiencesVocab'):
            return self.aq_parent.getAudiencesVocab()
        else:
            return ()

    security.declareProtected(CMFCorePermissions.View, 'getSubjectVocab')
    def getSubjectVocab(self):
        """Get subject (keywords) vocabulary"""
        catalog = getToolByName(self, 'portal_catalog')
        return catalog.uniqueValuesFor('Subject')

    security.declareProtected(CMFCorePermissions.View, 'Versions')
    def Versions(self):
        """method to display the versions in a nicer way
        """
        field = self.getField('versions')
        if field:
            return ", ".join(field.get(self))
        else:
            return ''

    security.declareProtected(CMFCorePermissions.View, 'Audiences')
    def Audiences(self):
        """method to display the audiences in a nicer way
        """
        field = self.getField('audiences')
        if field:
            return ", ".join(field.get(self))
        else:
            return ''

    security.declareProtected(CMFCorePermissions.View, 'isOutdated')
    def isOutdated(self):
        """Check the current versions of the PHC root container against the
        versions of this item. If the version of this item is not in the list
        of current versions, return 1, else return 0. As a shortcircuit, if
        no "currentVersions" is defined, always return 0.
        """

        myVersions = self.getVersions()

        if not myVersions:
            return 0

        # Acquire current versions
        currentVersions = [x.decode('utf-8') for x in self.getCurrentVersions()]

        if not currentVersions:
            return 0

        for v in myVersions:
            if v in currentVersions:
                # Not outdated - we match one of the current versions
                return 0

        # Outdated - we didn't match anything
        return 1

    security.declareProtected(CMFCorePermissions.View, 'getRelatedItems') 
    def getRelatedItems(self):
        """method to fetch the referenced items in context of
           config and permissions
        """
        try:
            objs = [o for o in self.getField('relatedItems').get(self) 
                      if self.portal_membership.checkPermission('View', o)]
            return objs
        except:
            return []


def HideOwnershipFields(schema):
    """ Some PHC types should not have their own ownership metadata """
    for fname in ('creators', 'contributors', 'rights'):
        schema[fname].widget.visible = {'view':'invisible','edit':'invisible'}


class PHCContent(BrowserDefaultMixin, HistoryAwareMixin, PHCContentMixin):
    """A simple  mixin class to provide contentish functions
    archetype no schema defined"""


    security = ClassSecurityInfo()

    _at_rename_after_creation = True

    __implements__ = (HistoryAwareMixin.__implements__,)




from zope import event
from zope.interface import implements
from zope.lifecycleevent import ObjectModifiedEvent

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

import Products.CMFCore.permissions as CMFCorePermissions


from Products.PloneHelpCenter.config import *
from Products.PloneHelpCenter.interfaces import IHelpCenterHowTo
from schemata import HelpCenterItemSchemaNarrow
from PHCContent import PHCContentMixin, HideOwnershipFields, IHelpCenterContent

from Products.ATContentTypes.interfaces import IATFolder, IATDocument
from Products.ATContentTypes.content.document import \
    ATDocumentSchema, ATDocumentBase
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema
from Products.ATContentTypes.content.base import ATCTOrderedFolder


# Create a Frankenstein's monster: a hybrid of ATCT
# folder and document types.
# Since the document type is more complex, we'll use it for the base.

HowToSchema = ATDocumentSchema.copy() + ConstrainTypesMixinSchema + HelpCenterItemSchemaNarrow
HideOwnershipFields(HowToSchema)
finalizeATCTSchema(HowToSchema, folderish=True, moveDiscussion=False)
for key, attr in DEFAULT_CONTENT_TYPES.items():
    setattr(HowToSchema['text'], key, attr)


class HelpCenterHowTo(ATDocumentBase, PHCContentMixin, ATCTOrderedFolder):

    """A How-to is a document describing how to address a single, common 
    use-case or issue. You may add images and files as attachments.
    """
    
    __implements__ = (ATCTOrderedFolder.__implements__,
                      IATFolder,
                      ATDocumentBase.__implements__, 
                      IATDocument,
                      )
    
    implements( (IHelpCenterHowTo, IHelpCenterContent,) )

    isPrincipiaFolderish = True

    content_icon = 'howto_icon.gif'

    typeDescription= 'A How-to is a document describing how to address a single, common use-case or issue. You may add images and files as attachments.'
    typeDescMsgId  = 'description_edit_howto'


    schema = HowToSchema
    archetype_name = 'How-to'
    meta_type = 'HelpCenterHowTo'

    security = ClassSecurityInfo()

    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        """Check if the user has permission to select a default page on this
        (folderish) item, and the item is folderish.
        """
        return False



registerType(HelpCenterHowTo, PROJECTNAME)
